@@ -1,7 +1,7 @@
 /*
  * @(#)AbacusC.c
  *
- * Copyright 1992 - 2018  David A. Bagley, bagleyd AT verizon.net
+ * Copyright 1992 - 2020  David A. Bagley, bagleyd AT verizon.net
  *
  * All rights reserved.
  *
@@ -17,7 +17,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
- * Taken from a C++ group project where I was a lead contributer
+ * Taken from a C++ group project where I was a lead contributor
  * OOP Group4!
  */
 
@@ -120,7 +120,7 @@ m	memory ('m', ('0'-'9' | 'A'-'F'))
 n	bitwise negation
 o	degree mode
 p	pi
-q	clear everything (quit all calulations) [clear]
+q	clear everything (quit all calculations) [clear]
 r	radian mode
 s	sine
 t	tangent
@@ -1101,10 +1101,15 @@ convertStringToAbacus(AbacusWidget w, const char *string, int aux)
 	int num, factor, i, sign = 0, len = (int) strlen(string);
 	AbacusWidget *wap = &w, wa;
 
-	if (aux == 1 && w->abacus.leftAux != NULL)
+	if (aux == 1) {
+		if (w->abacus.leftAux == NULL)
+			return;
 		wap = (AbacusWidget *) w->abacus.leftAux;
-	else if (aux == 2 && w->abacus.rightAux != NULL)
+	} else if (aux == 2) {
+		if (w->abacus.rightAux == NULL)
+			return;
 		wap = (AbacusWidget *) w->abacus.rightAux;
+	}
 	wa = *wap;
 	if (aux == 0) {
 		clearRails(w);
@@ -1227,10 +1232,9 @@ convertStringBases(char *sb, char *buf, int displayBase, int base) {
 			}
 			i--;
 			numberString[j] = '\0';
-			num = convertToDecimal(displayBase,
-				numberString);
+			num = convertToDecimal(displayBase, numberString);
 			convertFromDouble(numberString, base, num);
-			(void) strncat(sb, numberString, CALC_STRING_SIZE - 1);
+			(void) strcat(sb, numberString);
 		} else {
 			sb = strAddChar(sb, buf[i]);
 		}
@@ -1259,7 +1263,8 @@ void calculate(AbacusWidget w, char * buffer, int aux)
 		(void) printf("conv buffer =%s\n", stringBuf);
 #endif
 	} else {
-		(void) strncpy(stringBuf, buffer, CALC_STRING_SIZE);
+		(void) strncpy(stringBuf, buffer, CALC_STRING_SIZE - 1);
+		stringBuf[CALC_STRING_SIZE - 1] = '\0';
 	}
 	resetExpression();
 	for (i = 0; i < strlen(stringBuf); i++) {
