@@ -61,6 +61,8 @@ extern void selectAbacus(AbacusWidget w, const int x, const int y);
 extern void releaseAbacus(AbacusWidget w, const int x, const int y);
 extern void clearAbacus(AbacusWidget w);
 extern void clearDecimalAbacus(AbacusWidget w);
+extern void undoAbacus(AbacusWidget w);
+extern void redoAbacus(AbacusWidget w);
 extern void complementAbacus(AbacusWidget w);
 extern void incrementAbacus(AbacusWidget w);
 extern void decrementAbacus(AbacusWidget w);
@@ -186,6 +188,20 @@ extern AbacusClassRec abacusClassRec;
 	{fillRectClipX(w,dr,c,dy,dx,sy,sx,o,wo,ws);} else \
 	{fillRectClipY(w,dr,c,dx,dy,sx,sy,o,wo,ws);}
 
+typedef struct _MoveRecord {
+	int aux, deck, rail, number;
+} MoveRecord;
+
+typedef struct _MoveStack {
+	MoveRecord move;
+	struct _MoveStack *previous, *next;
+} MoveStack;
+
+typedef struct _AbacusStack {
+	MoveStack *currMove, *lastMove, *firstMove;
+	int count;
+} AbacusStack;
+
 typedef struct _MuseumPart {
 	char        museum[MAX_MUSEUMS];	/* 2 char country code of museum */
 	Boolean     separateSlots;
@@ -243,7 +259,7 @@ typedef struct _AbacusPart {
 	Pixmap      bufferBead[4][2][2];
 	TimeVal     oldTime;
 	Boolean     rightToLeftAdd, rightToLeftMult;
-	int         step, carry[2], lower, upper, carryStep;
+	int         step, carry[2], lower, upper, carryStep, masterStep;
 	int         state; /* explain or carrying out operation */
 	int         reg, regCount, qDigit, qPosition, auxCount, primarySteps;
 	double      divisor;
@@ -385,6 +401,17 @@ extern void drawBeadRail(AbacusWidget w, int rail, Boolean highlight);
 extern int convertBaseToBottom(int base);
 extern int rootInt(int i, int n);
 
+extern void newMoves(AbacusStack *s);
+extern void deleteMoves(AbacusStack *s);
+extern void setMove(AbacusStack *s,
+	int aux, int deck, int rail, int number);
+extern void getMove(AbacusStack *s,
+	int *aux, int *deck, int *rail, int *number);
+extern int madeMoves(AbacusStack *s);
+extern void flushMoves(AbacusStack *s);
+extern int numMoves(AbacusStack *s);
+extern void scanMoves(FILE *fp, AbacusWidget w, int moves);
+extern void printMoves(FILE *fp, AbacusStack *s);
 extern void clearRails(AbacusWidget w);
 extern void clearDecimalRails(AbacusWidget w);
 extern void addBackAnomaly(char * buf, int anomaly, int shift, int base);
